
# q-expansion of newform 6336.2.b.e, downloaded from the LMFDB on 03 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6336
weight = 2
poly_data = [2, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[4159, [1, 0]], [4357, [1, 0]], [3521, [-1, 0]], [1729, [-1, 0]]]
aps_data = [[0, 0], [0, 0], [0, 0], [0, -2], [-3, 1], [0, 3], [2, 0], [0, -1], [0, -1], [-4, 0], [-6, 0], [6, 0], [-2, 0], [0, -1], [0, -3], [0, -6], [0, 8], [0, -3], [12, 0], [0, -9], [0, 6], [0, -8], [-6, 0], [0, -3], [-12, 0], [10, 0], [6, 0], [-12, 0], [0, -3], [0, -3], [0, -8], [-12, 0], [0, 3], [0, -5], [10, 0], [0, -4], [-14, 0], [24, 0], [12, 0], [-16, 0], [0, 6], [-2, 0], [0, -15], [0, -18], [-10, 0], [-24, 0], [0, -17], [0, 0], [-12, 0], [6, 0], [14, 0], [-24, 0], [0, 0], [0, -6], [0, 15], [-12, 0], [0, -6], [0, -16], [0, -3], [-10, 0], [0, 7], [-28, 0], [0, -5], [0, -1], [0, 0], [0, 6], [12, 0], [0, 18], [-6, 0], [0, -9], [0, 9], [12, 0], [24, 0], [0, 3], [12, 0], [0, 1], [0, 12], [18, 0], [0, -21], [0, -12], [0, -18], [10, 0], [0, 0], [24, 0], [0, 10], [0, -16], [0, -3], [0, -12], [16, 0], [18, 0], [0, -14], [-24, 0], [0, 0], [36, 0], [-12, 0], [0, 0], [0, -30], [0, -21], [0, 7], [0, 15], [0, -19], [-16, 0], [18, 0], [34, 0], [0, 1], [-20, 0], [0, -20], [34, 0], [0, 17], [0, 24], [0, -16], [0, -3], [0, 33], [-36, 0], [-30, 0], [0, -9], [-12, 0], [0, 31], [0, -12], [-30, 0], [14, 0], [0, -18], [-14, 0], [0, 6], [-24, 0], [-20, 0], [-30, 0], [0, 27], [48, 0], [0, -27], [0, -7], [36, 0], [18, 0], [-34, 0], [-10, 0], [0, 18], [0, -6], [0, -25], [0, 0], [-46, 0], [0, 31], [-10, 0], [24, 0], [30, 0], [-6, 0], [0, 25], [0, 39], [-14, 0], [-48, 0], [0, 13], [0, -33], [0, 9], [36, 0], [12, 0], [-36, 0], [0, -33], [0, 4], [0, -3], [0, -12], [38, 0], [0, 0], [34, 0], [0, 40], [0, 40], [0, -27], [0, 11], [-18, 0], [0, -33], [0, 24], [0, 24], [12, 0], [30, 0], [60, 0], [0, 18], [0, 0], [0, 27], [0, 41], [0, -36], [0, 38], [0, 27], [30, 0], [-36, 0], [22, 0], [26, 0], [0, 15], [0, -36], [0, 45], [-48, 0], [0, 24], [24, 0], [-34, 0], [-6, 0], [48, 0], [0, 0], [-18, 0], [0, -39], [0, 12], [-26, 0], [62, 0], [36, 0], [-22, 0], [0, -22], [22, 0], [0, -42], [0, -6], [0, 30], [-18, 0], [-12, 0], [10, 0], [24, 0], [0, -42], [0, -36], [-30, 0], [0, 34], [48, 0], [54, 0], [0, -2], [70, 0], [0, -27], [0, -12], [0, 39], [0, 8], [0, -15], [-48, 0], [42, 0], [0, 27], [0, -51], [0, 23], [0, 46], [12, 0], [-18, 0], [0, 41], [0, 22], [46, 0], [-36, 0], [-60, 0], [4, 0], [-8, 0], [0, 18], [0, 13], [0, 2], [0, 49], [-42, 0], [34, 0], [-62, 0], [48, 0], [-24, 0], [0, 48], [0, 19], [60, 0], [0, 27], [-22, 0], [0, 33], [-24, 0], [-14, 0], [6, 0], [66, 0], [0, -29], [0, -6], [0, 30], [0, 20], [-36, 0], [0, -27], [0, -51], [0, -33], [0, 0], [0, 36], [0, -33], [0, 53], [32, 0], [-18, 0], [12, 0], [26, 0], [0, -2], [0, -30], [-48, 0], [0, 12], [0, -33], [0, 48], [-78, 0], [36, 0], [0, 0], [-48, 0], [0, 9], [0, 17], [0, -31], [-60, 0], [-32, 0], [0, 0], [-22, 0], [0, 24], [0, -40], [38, 0], [30, 0], [0, -15], [10, 0], [-24, 0], [0, 30], [78, 0], [0, -35], [0, -42], [16, 0], [0, -10], [0, 10], [36, 0], [30, 0], [0, -36], [-86, 0], [0, -19], [0, -21], [-48, 0], [0, 30], [62, 0], [-36, 0], [48, 0], [0, 18], [0, -34], [60, 0], [8, 0], [-2, 0], [0, -47], [-32, 0], [-82, 0], [-24, 0], [14, 0], [-2, 0], [36, 0], [12, 0], [-60, 0], [56, 0], [0, -3], [0, -24], [0, 62], [-60, 0], [0, 1], [0, -50], [-6, 0], [2, 0], [32, 0], [0, -44], [-6, 0], [0, -39], [-68, 0], [-72, 0], [0, 66], [30, 0], [2, 0], [-60, 0], [-72, 0], [0, -54], [0, -31], [52, 0], [0, -54], [0, -28], [0, 9], [-14, 0], [0, 27], [-84, 0], [34, 0], [0, 27], [0, -33], [2, 0], [0, 23], [-60, 0], [-48, 0], [46, 0], [14, 0], [0, 4], [0, 6], [0, -52], [24, 0], [-36, 0], [-86, 0], [0, -34], [38, 0], [0, -12], [12, 0], [0, 60], [-70, 0], [0, 0], [0, -36], [0, -21], [0, -28], [10, 0], [0, 1], [0, 15], [18, 0], [-18, 0], [0, -35], [0, 69], [72, 0], [0, 0], [0, -42], [36, 0], [0, -61], [0, 60], [0, 22], [0, 57], [12, 0], [8, 0], [0, 39], [0, -27], [0, -26], [0, -9], [52, 0], [0, 14], [-18, 0], [-86, 0], [-24, 0], [0, -48], [0, 0], [86, 0], [0, -36], [0, 25], [0, 30], [0, 66], [24, 0], [78, 0], [0, 9], [-48, 0], [0, -36], [0, 33], [0, 39], [84, 0], [16, 0], [0, 6], [0, 12], [-26, 0], [84, 0], [-96, 0]]
